@@ -1,13 +1,15 @@
 import logging
 import threading
+import warnings
 from collections import namedtuple
-from typing import Callable, Dict, Optional, Type, Union, get_origin, get_args
+from typing import Callable, Dict, Optional, Type, Union, TypeVar
 
 import zmq
 from pydantic import ValidationError, BaseModel
 from pydantic.main import ROOT_KEY
 
 from openmodule.config import settings
+from openmodule.dispatcher import DummyExecutor
 from openmodule.messaging import get_sub_socket, get_pub_socket, receive_message_from_socket
 from openmodule.models.rpc import RPCResponse, RPCRequest
 from openmodule.rpc.common import channel_to_request_topic, channel_to_response_topic
@@ -44,17 +46,38 @@ class Filter:
         return None
 
 
+RequestType = TypeVar("RequestType")
+ResponseType = TypeVar("ResponseType")
+
+
 class RPCServer(object):
-    def __init__(self, context, config, filter_resource=True):
+    def __init__(self, context, config=None, *, filter_resource=True, executor=None):
+        """
+        :param: executor: The executor to use for scheduling messages. The executor is shutdown
+                          if the RPC Server is shutdown
+        """
+        if config:
+            warnings.warn(
+                "\n\nThe RPC Server no longer requires the config, but takes the openmodule global config instead. "
+                "Please remove the argument. Future versions will remove the argument and leat to an error",
+                DeprecationWarning, stacklevel=2
+            )
+
+        config = config or settings
         self.name = config.NAME
         self.sub = get_sub_socket(context, config)
         self.pub = get_pub_socket(context, config)
+        self.pub_lock = threading.Lock()
         self.handlers: Dict[tuple, HandlerEntry] = {}
         self.filters = []
         self.running = True
         self.log = logging.getLogger(self.__class__.__name__)
         self.thread = None
         self.resource = None
+        self.executor = executor or DummyExecutor()
+        # since zmq does prefix matching we keep a list of unique channel names here
+        # so we do not log a warning "unknown handler" if our channel is a prefix of a different channel
+        self.registered_channels = set()
         if filter_resource:
             if config.RESOURCE:
                 self.resource = config.RESOURCE
@@ -72,19 +95,27 @@ class RPCServer(object):
         """
         self.filters.append(Filter(filter=filter, channel=channel, type=type))
 
-    def shutdown(self, timeout=3):
+    def shutdown(self, timeout=10):
         self.running = False
+        self.executor.shutdown(wait=True)
         if self.thread:
             self.thread.join(timeout=timeout)
+        else:
+            with self.pub_lock:
+                self.pub.close()
+            self.sub.close()
 
-    def register_handler(self, channel: str, type: str, request_class: Type[BaseModel], response_class: Type[BaseModel],
-                         handler: Callable, register_schema=True):
+    def register_handler(self, channel: str, type: str,
+                         request_class: Type[RequestType],
+                         response_class: Type[ResponseType],
+                         handler: Callable[[RequestType, BaseModel], Union[ResponseType, Dict, None]],
+                         register_schema=True):
         """
         :param channel: rpc channel you want to subscribe to, patterns/wildcards are not supported
         :param type: the request type
         :param request_class: Class definition of the request
         :param response_class: Class definition of the response
-        :param handler: request handler of the form (request:dict, meta:dict) -> any
+        :param handler: request handler of the form (request:Instance of request_class, full_message) -> Instance of response_class
                         if the handler returns a dict which contains the key "status",
                         then the value of "status" will be used as the rpc response status.
                         There are no other reserved keys except "status".
@@ -96,6 +127,7 @@ class RPCServer(object):
         self.log.debug("register handler {}:{} -> {}".format(channel, type, handler))
         self.sub.subscribe(channel_to_request_topic(channel.encode("ascii")))
         self.handlers[(channel, type)] = HandlerEntry(request_class, response_class, handler)
+        self.registered_channels = set(x[0] for x in self.handlers.keys())
         if register_schema:
             Schema.save_rpc(channel, type, request_class, response_class, handler)
 
@@ -171,6 +203,48 @@ class RPCServer(object):
             self.log.warning("no handler found for {}:{}".format(channel, message.type))
             return None
 
+    def _process_rpc_message(self, topic, message):
+        try:
+            channel = self._channel_from_topic(topic)
+
+            # this filters prefix-matched topics from other channels early
+            if channel not in self.registered_channels:
+                return
+
+            try:
+                message = RPCRequest(**message)
+                message_type = message.type
+                rpc_id = message.rpc_id
+            except ValidationError as e:
+                message_type = "unknown"
+                rpc_id = None
+                response = {"status": "validation_error", "exception": e.json()}
+            else:
+                response = self.process_rpc(channel, message)
+
+            if response is not None:
+                self.log.debug("response: {}".format(response))
+                if "status" not in response:
+                    response["status"] = "ok"
+
+                result = RPCResponse(
+                    type=message_type,
+                    rpc_id=rpc_id,
+                    response=response,
+                    name=self.name
+                )
+                with self.pub_lock:
+                    result.publish_on_topic(
+                        self.pub,
+                        channel_to_response_topic(channel.encode("ascii")),
+                    )
+        except Exception:
+            self.log.exception("Unknown exception while processing an rpc. No answer could was sent to the caller.")
+        except zmq.ContextTerminated:
+            self.log.warning(
+                "context terminated while RPC request was being processed. The result might not have been sent."
+            )
+
     def run(self):
         poller = zmq.Poller()
         poller.register(self.sub, zmq.POLLIN)
@@ -182,35 +256,12 @@ class RPCServer(object):
                 topic, message = receive_message_from_socket(self.sub)
                 if topic is None:
                     continue
-                channel = self._channel_from_topic(topic)
-                try:
-                    message = RPCRequest(**message)
-                    message_type = message.type
-                    rpc_id = message.rpc_id
-                except ValidationError as e:
-                    message_type = "unknown"
-                    rpc_id = None
-                    response = {"status": "validation_error", "exception": e.json()}
-                else:
-                    response = self.process_rpc(channel, message)
-
-                if response is not None:
-                    self.log.debug("response: {}".format(response))
-                    if "status" not in response:
-                        response["status"] = "ok"
-
-                    result = RPCResponse(
-                        type=message_type,
-                        rpc_id=rpc_id,
-                        response=response,
-                        name=self.name
-                    )
-                    result.publish_on_topic(
-                        self.pub,
-                        channel_to_response_topic(channel.encode("ascii")),
-                    )
+                if self.running:
+                  # someone may have stopped our server while we were receiving a message
+                  self.executor.submit(self._process_rpc_message, topic, message)
         except zmq.ContextTerminated:
             pass
         finally:
             self.sub.close()
-            self.pub.close()
+            with self.pub_lock:
+                self.pub.close()
