@@ -6,7 +6,7 @@ from openmodule.models.base import Gateway, ZMQMessage
 from openmodule.utils.io import IoListener
 from openmodule_test.core import OpenModuleCoreTestMixin
 from openmodule_test.eventlistener import MockEvent
-from openmodule_test.io import IoSimulator, generate_example_states
+from openmodule_test.io_simulator import IoSimulator, generate_example_states
 
 
 class IoTest(OpenModuleCoreTestMixin, TestCase):
@@ -14,7 +14,7 @@ class IoTest(OpenModuleCoreTestMixin, TestCase):
         super().setUp()
         self.io = IoListener(self.core.messages)
         io_init_event = MockEvent()
-        self.core.messages.register_handler(b"io", ZMQMessage, handler=io_init_event)
+        self.core.messages.register_handler(b"io", ZMQMessage, handler=io_init_event, match_type=False)
         self.wait_for_dispatcher(self.core.messages)
 
         self.io_sim = IoSimulator(generate_example_states(count=18), emit=lambda x: self.zmq_client.send(b"io", x))
@@ -40,7 +40,7 @@ class IoTest(OpenModuleCoreTestMixin, TestCase):
         self.io.add_listener_edge_none(listen_to=pin, callback=on_no_edge)
         self.io_sim.emit_current_pin_state(pin)
         on_no_edge.wait_for_call()
-        on_no_edge.reset_all_call_counts()
+        on_no_edge.reset_all_mocks()
         self.io_sim.change_pin_sate(pin)
         with self.assertRaises(TimeoutError):
             on_no_edge.wait_for_call(timeout=1)
@@ -52,10 +52,10 @@ class IoTest(OpenModuleCoreTestMixin, TestCase):
         self.io.add_listener_edge_none(listen_to=pin, callback=on_all_messages)
         self.io_sim.change_pin_sate(pin)
         on_all_messages.wait_for_call()
-        on_all_messages.reset_all_call_counts()
+        on_all_messages.reset_all_mocks()
         self.io_sim.change_pin_sate(pin)
         on_all_messages.wait_for_call()
-        on_all_messages.reset_all_call_counts()
+        on_all_messages.reset_all_mocks()
         self.io_sim.emit_current_pin_state(pin)
         on_all_messages.wait_for_call()
 
@@ -65,7 +65,7 @@ class IoTest(OpenModuleCoreTestMixin, TestCase):
         self.io.add_listener_edge_any(listen_to=pin, callback=on_any_edge)
         self.io_sim.change_pin_sate(pin)
         on_any_edge.wait_for_call()
-        on_any_edge.reset_all_call_counts()
+        on_any_edge.reset_all_mocks()
         self.io_sim.emit_current_pin_state(pin)
         with self.assertRaises(TimeoutError):
             on_any_edge.wait_for_call(timeout=1)
@@ -83,7 +83,7 @@ class IoTest(OpenModuleCoreTestMixin, TestCase):
         self.io.add_listener_edge_rising(listen_to=pin, callback=on_rising_edge)
         self.io_sim.set_pin_high(pin)
         on_rising_edge.wait_for_call()
-        on_rising_edge.reset_all_call_counts()
+        on_rising_edge.reset_all_mocks()
         self.io_sim.set_pin_low(pin)
         with self.assertRaises(TimeoutError):
             on_rising_edge.wait_for_call(timeout=1)
@@ -95,21 +95,26 @@ class IoTest(OpenModuleCoreTestMixin, TestCase):
         self.io_sim.set_pin_high(pin)
         with self.assertRaises(TimeoutError):
             on_falling_edge.wait_for_call(timeout=1)
-        on_falling_edge.reset_all_call_counts()
+        on_falling_edge.reset_all_mocks()
         self.io_sim.set_pin_low(pin)
         on_falling_edge.wait_for_call()
 
     def test_listen_to_gateway(self):
         on_any_edge = MockEvent()
         gw = list(self.io_sim.get_pin_states().values())[0].gateway
-        gateway = Gateway(gate=gw.gate, direction=gw.direction)
+
+        # check that the gateway direction does not matter for the filter
+        # by explicitly setting sthe wrong direction
+        self.assertEqual(gw.direction, "in")
+        gateway = Gateway(gate=gw.gate, direction="out")
+
         self.io.add_listener_edge_any(listen_to=gateway, callback=on_any_edge)
         self.io_sim.set_pin_high(list(self.io_sim.get_pin_states().keys())[0])
         on_any_edge.wait_for_call()
-        on_any_edge.reset_all_call_counts()
+        on_any_edge.reset_all_mocks()
         self.io_sim.set_pin_high(list(self.io_sim.get_pin_states().keys())[1])
         on_any_edge.wait_for_call()
-        on_any_edge.reset_all_call_counts()
+        on_any_edge.reset_all_mocks()
         self.io_sim.set_pin_high(list(self.io_sim.get_pin_states().keys())[5])
         with self.assertRaises(TimeoutError):
             on_any_edge.wait_for_call(timeout=1)
