@@ -1,7 +1,8 @@
 import logging
 import time
+from concurrent.futures import ThreadPoolExecutor
 from functools import partial
-from typing import Union
+from typing import Union, Callable
 from unittest import TestCase
 from uuid import uuid4
 
@@ -87,7 +88,7 @@ class TestRPCResponse2(OpenModuleModel):
 
 
 class RPCServerTestCase(RPCServerTestMixin):
-    rpc_channels = ["channel", "channel2"]
+    rpc_channels = ["channel", "channel2", "a", "ab"]
     topics = ["test"]
 
     server: RPCServer
@@ -95,7 +96,7 @@ class RPCServerTestCase(RPCServerTestMixin):
     def setUp(self):
         super().setUp()
         self.called_types = {}
-        self.server = RPCServer(self.zmq_context(), self.zmq_config())
+        self.server = RPCServer(self.zmq_context())
         self.server_thread = self.server.run_as_thread()
 
     def tearDown(self):
@@ -370,6 +371,25 @@ class RPCServerTestCase(RPCServerTestMixin):
         with self.assertRaises(TimeoutError):
             self.rpc("channel", "test-type", {}, resource="test-2")
 
+    def test_prefix_matched_topic(self):
+        def handler1(*_, **__):
+            """ test rpc handler"""
+            logging.info("handler 1")
+            return TestRPCResponse()
+
+        self.server.register_handler("a", "test-type", TestRPCRequest, TestRPCResponse, handler1)
+        self.wait_for_rpc_server(self.server)
+
+        with self.assertLogs() as cm:
+            # we use assertLogs here to capture log output, but we would need assertNoLogs from python3.10
+            # so we log some dummy message and check assertNotIn afterwards
+            logging.info("some-log-so assert works")
+
+            with self.assertRaises(TimeoutError):
+                self.rpc("ab", "test-type", {})
+
+        self.assertNotIn("no handler found", str(cm.output))
+
     def test_union_response_parsing(self):
         """
         tests that the rpc server correctly response when Union types are used as responses
@@ -482,56 +502,142 @@ class RpcClientTest(RPCServerTestMixin, OpenModuleCoreTestMixin):
 
     def setUp(self):
         super().setUp()
-        self.server = RPCServer(config=self.zmq_config(), context=self.zmq_context(), filter_resource=False)
+        self.server = RPCServer(context=self.zmq_context(), filter_resource=False)
         self.server.register_handler("channel", "test", TestRPCRequest, TestRPCResponse2, self.handler1)
         self.server.register_handler("channel", "timeout", TestRPCRequest, TestRPCResponse2, self.timeout_handler)
         self.server.run_as_thread()
         self.wait_for_rpc_server(self.server)
 
-        self.client = RPCClient(self.core, channels=[b"channel"])
+        self.client = RPCClient(self.core.messages, channels=[b"channel"], _warn=False)
         self.wait_for_dispatcher(self.core.messages)
 
     def test_blocking(self):
-        result = self.client.rpc(b"channel", "test", {}, timeout=1)
+        result = self.client.rpc(b"channel", "test", EmptyModel(), timeout=1)
         self.assertEqual("ok", result.status)
 
         with self.assertRaises(TimeoutError):
-            result = self.client.rpc(b"channel", "timeout", {}, timeout=1)
+            self.client.rpc(b"channel", "timeout", EmptyModel(), timeout=1)
 
     def test_non_blocking(self):
-        result = self.client.rpc(b"channel", "test", {}, timeout=1, blocking=False)
-        self.assertEqual("ok", result.status)
+        result = self.client.rpc(b"channel", "test", EmptyModel(), timeout=1, blocking=False)
+        self.assertFalse(None, result.done())
         self.assertEqual(None, result.response)
+
         time.sleep(1)
-        result = self.client.check_result(result.rpc_id)
-        self.assertIsNotNone(result.response)
+
+        self.assertTrue(result.done())
+        result = result.result()
+        self.assertEqual("ok", result.status)
+        self.assertIn("some_payload", result.response)
+
+    def test_non_blocking_via_result(self):
+        result = self.client.rpc(b"channel", "test", EmptyModel(), timeout=1, blocking=False)
+        result = result.result()
+        self.assertEqual("ok", result.status)
+        self.assertIn("some_payload", result.response)
+
+    def test_non_blocking_timeout(self):
+        result = self.client.rpc(b"channel", "timeout", EmptyModel(), timeout=1, blocking=False)
+        with self.assertRaises(TimeoutError):
+            result.result(timeout=1)
+
+    def test_non_blocking_higher_timeout_than_at_request(self):
+        result = self.client.rpc(b"channel", "timeout", EmptyModel(), timeout=1, blocking=False)
+        with self.assertRaises(TimeoutError):
+            with self.assertWarns(UserWarning) as cm:
+                result.result(timeout=2)
+        self.assertIn("cannot extend", str(cm.warnings[0].message))
+
+    def test_non_blocking_no_timeout(self):
+        result = self.client.rpc(b"channel", "timeout", EmptyModel(), timeout=4, blocking=False)
+        result = result.result()
+        self.assertEqual("ok", result.status)
+        self.assertIn("some_payload", result.response)
+
+    def test_proactive_timeout(self):
+        """
+        tests a case where the rpc result is cehcked after the timeout, in this case
+        we time-out immediately
+        """
+        result = self.client.rpc(b"channel", "timeout", EmptyModel(), timeout=1, blocking=False)
+        time.sleep(2)
+        start = time.time()
+        with self.assertRaises(TimeoutError):
+            result.result()
+        duration = time.time() - start
+        self.assertTrue(duration < 0.5)
+
+    def test_maximum_wait_time(self):
+        """
+        tests that we can atmost wait for the maximum timeout since the *request was sent*
+        because the cleanup will clean the message internally anways, there is no point in
+        waiting for a longer duration
+        """
+        result = self.client.rpc(b"channel", "does-not-exist", EmptyModel(), timeout=4, blocking=False)
+        time.sleep(2)
+        start = time.time()
+        with self.assertRaises(TimeoutError):
+            result.result()
+        duration = time.time() - start
+        # we waited 2 seconds before starting the result() call
+        # thus the result call can only wait ~2 seconds, since the total timeout is 4 seconds
+        self.assertTrue(1.5 < duration < 2.5)
+
+    def test_no_proactive_timeout_if_received_in_time(self):
+        """
+        this test ensures that if we call .result() _after_ the timeout,
+        but the response was received _within_ the timeout, that the
+        result is still retrievable
+        """
+        result = self.client.rpc(b"channel", "test", EmptyModel(), timeout=1, blocking=False)
+        time.sleep(3)
+        self.assertTrue(result.done())
+        result = result.result()
         self.assertEqual("ok", result.status)
         self.assertIn("some_payload", result.response)
 
 
-class RpcClientResourceTest(RPCServerTestMixin, OpenModuleCoreTestMixin):
-    rpc_channels = ["channel"]
+class TestRPCResponseNotSerializable(OpenModuleModel):
+    __test__ = False
+    some_payload: Callable
+
+
+class RPCServerMultithreadingTest(OpenModuleCoreTestMixin, TestCase):
+    """
+    serialization errors were not captured in multithreaded rpc server
+    this testcase ensures that this is not the case anymore
+    """
+
+    rpc_channels = ["test"]
+    topics = ["sentry"]
+    init_kwargs = {"sentry": True}
+
+    def raise_exception_handler(self, _, __):
+        """
+        returns a model which cannot be serialized to json
+        """
+        return {"some_payload": self.rpc}
 
     def setUp(self):
         super().setUp()
-        self.server = RPCServer(config=self.zmq_config(), context=self.zmq_context(), filter_resource=True)
-        self.server.register_handler("channel", "test", TestRPCRequest, TestRPCResponse2, RpcClientTest.handler1)
-        self.server.register_handler("channel", "timeout", TestRPCRequest, TestRPCResponse2,
-                                     RpcClientTest.timeout_handler)
+
+        self.executor = ThreadPoolExecutor(max_workers=3)
+        self.server = RPCServer(self.zmq_context(), executor=self.executor)
+        self.server.register_handler("test", "test", EmptyModel, TestRPCResponseNotSerializable,
+                                     self.raise_exception_handler, register_schema=False)
         self.server.run_as_thread()
-        self.wait_for_rpc_server(self.server)
 
-        self.client = RPCClient(self.core, channels=[b"channel"])
-        self.wait_for_dispatcher(self.core.messages)
+    def tearDown(self):
+        super().tearDown()
+        self.server.shutdown()
 
-    def test_resource_filter(self):
-        result = self.client.rpc(b"channel", "test", {}, timeout=1)
-        self.assertEqual("ok", result.status)
+    def test_executor_logs_exceptions(self):
+        """
+        ensures that when using a multiprocessing executor exceptions are logged to logging and sentry
+        """
+        with self.assertLogs(level=logging.ERROR) as cm:
+            self.rpc("test", "test", EmptyModel(), receive_response=False)
+            self.zmq_client.wait_for_message_on_topic(b"sentry")
+        self.assertTrue(cm.output)
 
-        with override_context(RESOURCE="asdf"):
-            with self.assertRaises(TimeoutError):
-                self.client.rpc(b"channel", "test", {}, timeout=1)
 
-        with override_context(RESOURCE=""):
-            result = self.client.rpc(b"channel", "test", {}, timeout=1)
-            self.assertEqual("ok", result.status)
