@@ -1,9 +1,7 @@
-from unittest import TestCase
-
 import json
 from textwrap import dedent
+from unittest import TestCase
 
-from openmodule.config import override_settings
 from openmodule.models.backend import CountMessage, AccessRequest, Access, AccessCategory, check_recurrence
 from openmodule.models.base import Gateway
 from openmodule.rpc.server import RPCServer
@@ -133,7 +131,6 @@ class ModelTest(TestCase):
         self.assertIn("not a valid enumeration", str(e.exception))
 
 
-@override_settings(USER_PREFIX="backend")
 class NotImplementedTestCase(BackendTestMixin, TestCase):
     backend_class = Backend
 
@@ -146,7 +143,7 @@ class NotImplementedTestCase(BackendTestMixin, TestCase):
         with self.assertRaises(NotImplementedError):
             self.backend.check_access(None)
 
-@override_settings(USER_PREFIX="backend")
+
 class FunctionTest(BackendTestMixin, TestCase):
     backend_class = TestBackend
 
@@ -203,7 +200,7 @@ class FunctionTest(BackendTestMixin, TestCase):
         message = self.check_out_medium_changed(transaction_id)
         self.assertEqual(None, self.messages_by_transaction.get(transaction_id))
         self.assertEqual("medium_type_changed", message.error)
-        self.assertNotEqual(None, message.previous_id)
+        self.assertNotEqual(None, message.previous_medium_id)
         self.assertNotEqual(None, message.previous_medium_type)
 
         transaction_id = self.check_in(send=False)
@@ -216,10 +213,10 @@ class FunctionTest(BackendTestMixin, TestCase):
         message = self.check_out_medium_id_changed(transaction_id)
         self.assertEqual(None, self.messages_by_transaction.get(transaction_id))
         self.assertEqual("medium_id_changed", message.error)
-        self.assertNotEqual(None, message.previous_id)
+        self.assertNotEqual(None, message.previous_medium_id)
 
         transaction_id = self.check_in(send=False)
-        medium_id = self.messages_by_transaction[transaction_id].id
+        medium_id = self.messages_by_transaction[transaction_id].medium_id
         with self.assertRaises(Exception) as e:
             self.check_out_medium_id_changed(transaction_id, medium_id=medium_id)
         self.assertIn("medium_id needs to change", str(e.exception))
@@ -237,19 +234,18 @@ class FunctionTest(BackendTestMixin, TestCase):
         self.assertIn("user needs to change", str(e.exception))
 
     def test_access(self):
-        result = self.check_auth()
+        result = self.check_auth(medium_id="GARIVO1", medium_type="lpr")
         self.assertEqual(True, result.success)
         self.assertNotEqual([], result.medium.accesses)
 
-        result = self.check_auth(gate="empty")
+        result = self.check_auth(medium_id="GARIVO1", medium_type="lpr", gateway=Gateway(gate="empty", direction="in"))
         self.assertEqual(True, result.success)
         self.assertEqual([], result.medium.accesses)
 
-        result = self.check_auth(gate="error")
+        result = self.check_auth(medium_id="GARIVO1", medium_type="lpr", gateway=Gateway(gate="error", gateway="in"))
         self.assertEqual(False, result.success)
 
 
-@override_settings(USER_PREFIX="backend")
 class RpcTest(BackendTestMixin, RPCServerTestMixin, TestCase):
     rpc_channels = ["backend"]
     topics = [b"backend", b"count", b"healthz"]
@@ -257,7 +253,7 @@ class RpcTest(BackendTestMixin, RPCServerTestMixin, TestCase):
 
     def setUp(self):
         super().setUp()
-        self.server = RPCServer(config=self.zmq_config(), context=self.zmq_context())
+        self.server = RPCServer(context=self.zmq_context())
         self.server.run_as_thread()
         self.backend.register_rpcs(self.server)
         self.wait_for_rpc_server(self.server)
@@ -272,7 +268,7 @@ class RpcTest(BackendTestMixin, RPCServerTestMixin, TestCase):
         request = AccessRequest(gateway=gateway, name=self.core.config.NAME, medium_type="lpr", id="medium_id")
         response = self.rpc("backend", "auth", request=request.dict())
         self.assertRPCSuccess(response)
-        self.assertIn(request.id, self.backend.accessed)
+        self.assertIn(request.medium_id, self.backend.accessed)
 
     def send_count_message(self, gate="gate", direction="in"):
         transaction_id = self.create_count_message(direction=direction, gate=gate)
@@ -295,7 +291,7 @@ class RpcTest(BackendTestMixin, RPCServerTestMixin, TestCase):
         self.backend.wait_for_message_process()
         self.assertEqual(1, self.backend.checked_in)
 
-        MockEvent.reset_all_call_counts()
+        MockEvent.reset_all_mocks()
         with self.assertLogs() as cm:
             self.send_count_message(gate="error")
             self.backend.wait_for_message_process()
@@ -307,14 +303,14 @@ class RpcTest(BackendTestMixin, RPCServerTestMixin, TestCase):
         self.backend.wait_for_message_process()
         self.assertEqual(1, self.backend.checked_in)
 
-        MockEvent.reset_all_call_counts()
+        MockEvent.reset_all_mocks()
         with self.assertLogs() as cm:
             self.send_count_message(gate="error", direction="out")
             self.backend.wait_for_message_process()
         self.assertEqual(1, self.backend.checked_in)
         self.assertIn("error in check_out", cm.output[0])
 
-        MockEvent.reset_all_call_counts()
+        MockEvent.reset_all_mocks()
         self.send_count_message(direction="out")
         self.backend.wait_for_message_process()
         self.assertEqual(0, self.backend.checked_in)
